import * as React from "react";
import * as PropTypes from "prop-types";
import { InputState } from "./types/InputState";
export interface FieldProps {
    value: InputState<any>;
    /** callback for whenever the Field changes. Please note the following:
     * - contains a React FormEvent<> object when using an input type="file" */
    onChange?: (value: any) => void;
    debounce?: number;
    onBlur?: boolean;
    validate?: (value: any) => number;
    /** If type is number, the numeric value must be >= min. Else, the string length must be >= to min*/
    min?: number;
    /** If type is number, the numeric value must be <= max. Else, the string length must be <= to max */
    max?: number;
    /** Input must match regex expression1 */
    regex?: string | RegExp;
    /** Must have a value or Field is invalid */
    required?: boolean;
    /** Remove whitespace (left and right) from input */
    trimWhitespace?: boolean;
    children?: React.ReactNode;
}
export interface FieldState {
    /** value displayed in the input. Tracked separately to support things like trimWhitespace
     * where value in InputState does not match the value shown on screen */
    value: any;
    /** Shadow copy of props value to support cases like debounce where FormState should not immediate update*/
    input: InputState<any>;
}
/**
 * Field wraps around a single! input component that implements the onChange/value controlled component contract.
 * Must be inside a <Form> component.
 */
export declare class Field extends React.Component<FieldProps, FieldState> {
    static contextTypes: {
        updateField: PropTypes.Requireable<any>;
        updateForm: PropTypes.Requireable<any>;
    };
    private impl;
    private debounceTimer;
    constructor(props: FieldProps);
    render(): React.ReactElement<any>;
    componentWillMount(): void;
    componentWillReceiveProps(nextProps: FieldProps): void;
    componentDidUpdate(prevProps: FieldProps, prevState: FieldState): void;
    protected handleBlur(): void;
    private handleUpdateField(state, isFirstRender?);
    private updateForm(value?, isFirstRender?);
}
