"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
var InvalidField_1 = require("./field/impl/InvalidField");
var RadioButtonInputField_1 = require("./field/impl/RadioButtonInputField");
var NumberInputField_1 = require("./field/impl/NumberInputField");
var TextInputField_1 = require("./field/impl/TextInputField");
var EmailInputField_1 = require("./field/impl/EmailInputField");
var CheckboxInputField_1 = require("./field/impl/CheckboxInputField");
var CustomField_1 = require("./field/impl/CustomField");
var SelectinputField_1 = require("./field/impl/SelectinputField");
var util_1 = require("./util");
var DateinputField_1 = require("./field/impl/DateinputField");
var helpers_1 = require("./field/helpers");
var FileInputField_1 = require("./field/impl/FileInputField");
/**
 * Field wraps around a single! input component that implements the onChange/value controlled component contract.
 * Must be inside a <Form> component.
 */
var Field = (function (_super) {
    __extends(Field, _super);
    function Field(props) {
        var _this = _super.call(this, props) || this;
        _this.debounceTimer = -1;
        _this.handleUpdateField = _this.handleUpdateField.bind(_this);
        _this.updateForm = _this.updateForm.bind(_this);
        _this.handleBlur = _this.handleBlur.bind(_this);
        _this.state = {
            value: props.value.value,
            input: undefined,
        };
        return _this;
    }
    Field.prototype.render = function () {
        return this.impl.render();
    };
    Field.prototype.componentWillMount = function () {
        var _this = this;
        // Check for error with the children. Return a render that renders null if errors.
        var isValid = isFieldChildrenValid(this.props);
        if (!isValid) {
            this.impl = new InvalidField_1.InvalidField();
            return;
        }
        var getProps = function () { return _this.props; };
        var getState = function () { return _this.state; };
        var getContext = function () { return _this.context; };
        this.impl = createFieldImpl(this.props.children, this.handleUpdateField, this.handleBlur, getProps, getState, getContext);
        function isFieldChildrenValid(props) {
            var isValid = true;
            if (props.children instanceof Array && props.children.length > 1) {
                // Only time it is valid to have multiple children is when they belong to a radio button
                for (var i = 0; i < props.children.length; i++) {
                    var child = props.children[i];
                    if (!child || child.type === "label")
                        continue;
                    if (child.type !== "input" || child.props.type !== "radio") {
                        console.error("Detected a " + child.props.type + " <input> element. If <Field> contains mutliple elemnets, they must be type=\"radio\" or \"label\"");
                        isValid = false;
                        break;
                    }
                }
            }
            else if (props.children == null || (props.children instanceof Array && props.children.length === 0)) {
                console.error("Need child inside a <Field> component");
                isValid = false;
            }
            return isValid;
        }
        function createFieldImpl(children, handleUpdateField, handleBlur, getProps, getState, getContext) {
            var name = getProps().value.name;
            var value = getProps().value.value;
            // If children is an an array, then it must be a radio button group
            if (children instanceof Array && children.length > 1) {
                return new RadioButtonInputField_1.RadioButtonInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
            }
            var child = React.Children.only(children);
            if (child.type === "input") {
                var inputType = child.props.type;
                switch (inputType) {
                    case "number":
                        return new NumberInputField_1.NumberInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "search":
                    case "text":
                    case "password":
                        return new TextInputField_1.TextInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "email":
                        return new EmailInputField_1.EmailInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "checkbox":
                        return new CheckboxInputField_1.CheckboxInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "date":
                        return new DateinputField_1.DateInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "file":
                        return new FileInputField_1.FileInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "color":
                    case "image":
                    case "month":
                    case "range":
                    case "tel":
                    case "time":
                    case "url":
                    case "week":
                        console.warn("type=\"" + inputType + "\" is not implemented");
                        return new TextInputField_1.TextInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "radio":// Should never be encountered here becas
                        console.error("bug: couldn't detect radio button input");
                        return new InvalidField_1.InvalidField();
                    case "hidden":
                        console.warn("type=\"hidden\" inputs are not supported. Remove <Field> wrapper");
                        return new TextInputField_1.TextInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
                    case "reset":
                    case "button":
                    case "submit":
                        // buttons are invalid form inputs
                        console.warn("Do not use buttons inside of <Field> components");
                        return new InvalidField_1.InvalidField();
                }
                util_1.exhaustiveSwitch(inputType);
            }
            if (child.type === "select") {
                return new SelectinputField_1.SelectInputField(handleUpdateField, handleBlur, getProps, getState, value, name);
            }
            // Unknown input element. Assuming custom component that implements vlaue/onChange interface.
            return new CustomField_1.CustomField(handleUpdateField, getProps, getState, getContext, value, name);
        }
    };
    Field.prototype.componentWillReceiveProps = function (nextProps) {
        this.setState({
            value: nextProps.value.value,
            input: nextProps.value,
        });
    };
    Field.prototype.componentDidUpdate = function (prevProps, prevState) {
        this.impl.propsUpdate();
    };
    Field.prototype.handleBlur = function () {
        if (helpers_1.isAsync(this.props)) {
            // cancel any ongoing debounces since the update has already triggered
            clearTimeout(this.debounceTimer);
            this.debounceTimer = -1;
            this.updateForm();
        }
    };
    Field.prototype.handleUpdateField = function (state, isFirstRender) {
        if (isFirstRender === void 0) { isFirstRender = false; }
        this.setState(state);
        this.context.updateField(state.input.name, state.input);
        // Check for debounce. If numeric then install timeout. if onBlur, just exit function without updating Form.
        clearTimeout(this.debounceTimer);
        if (helpers_1.isAsync(this.props) && !isFirstRender) {
            if (helpers_1.isDebounced(this.props))
                this.debounceTimer = setTimeout(this.updateForm, this.props.debounce);
        }
        else {
            this.updateForm(state.input.value, isFirstRender);
        }
    };
    Field.prototype.updateForm = function (value, isFirstRender) {
        // When called synchronously, this.state does not have the next value
        if (this.props.onChange && !isFirstRender)
            this.props.onChange(value !== undefined ? value : this.state.input.value);
        this.context.updateForm();
    };
    Field.contextTypes = {
        updateField: PropTypes.func,
        updateForm: PropTypes.func,
    };
    return Field;
}(React.Component));
exports.Field = Field;
