import * as React from "react";
import * as PropTypes from "prop-types";
import { BaseForm } from "./types/BaseForm";
import { InputState } from "./types/InputState";
export interface FormProps {
    /** If true, the default form behaviour (POST formdata) is prevented.
     * onSubmit can then be implemented to perform whatever action you like. */
    preventAction?: boolean;
    formState: BaseForm;
    /** Classname to apply to <form> element */
    className?: string;
    /** Callback executed when the form is submitted.
     * Return false to stop submit action */
    onSubmit?: (formState: BaseForm) => boolean;
    onChange(formState: BaseForm): void;
}
export declare class Form extends React.Component<FormProps, object> {
    static childContextTypes: {
        updateField: PropTypes.Requireable<any>;
        updateForm: PropTypes.Requireable<any>;
    };
    formRef: HTMLFormElement | null;
    private formState;
    /** Used to batch updates when mutliple Field children change at the same time. */
    private updateFormTimer;
    constructor(props: FormProps);
    getChildContext(): {
        updateField: (name: string, nextInputState: InputState<any>) => void;
        updateForm: () => void;
    };
    render(): JSX.Element;
    componentWillReceiveProps(nextProps: FormProps): void;
    private handleSubmit(event);
    private updateForm();
    private batchedUpdateForm();
    private updateField(name, nextInputState);
}
