"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
var Form = (function (_super) {
    __extends(Form, _super);
    function Form(props) {
        var _this = _super.call(this, props) || this;
        /** Used to batch updates when mutliple Field children change at the same time. */
        _this.updateFormTimer = -1;
        _this.updateForm = _this.updateForm.bind(_this);
        _this.handleSubmit = _this.handleSubmit.bind(_this);
        _this.updateField = _this.updateField.bind(_this);
        _this.batchedUpdateForm = _this.batchedUpdateForm.bind(_this);
        _this.formState = __assign({}, props.formState);
        return _this;
    }
    Form.prototype.getChildContext = function () {
        return {
            updateField: this.updateField,
            updateForm: this.updateForm,
        };
    };
    Form.prototype.render = function () {
        var _this = this;
        var className = this.props.className || undefined;
        return (React.createElement("form", { ref: function (ref) { return _this.formRef = ref; }, method: this.props.preventAction ? undefined : "post", noValidate: true, onSubmit: this.handleSubmit, className: className }, this.props.children));
    };
    Form.prototype.componentWillReceiveProps = function (nextProps) {
        this.formState = nextProps.formState === this.formState ? this.formState : __assign({}, nextProps.formState);
    };
    Form.prototype.handleSubmit = function (event) {
        if (this.formState.isSubmitted) {
            event.preventDefault();
            console.warn("Submit was triggered on a form that was already submitted. Preventing action");
            return;
        }
        if (!this.formState.isValid) {
            event.preventDefault();
            console.warn("Could not submit. Form is invalid");
            return;
        }
        // Check if the user wants to bail out of the submit
        var canSubmit = (this.props.onSubmit !== undefined) ? this.props.onSubmit(this.formState) : true;
        if (!canSubmit) {
            event.preventDefault();
            return;
        }
        this.formState.isSubmitted = true;
        this.updateForm();
        // Prevent default form behaviour will POST to the current url or a given url
        if (this.props.preventAction)
            event.preventDefault();
    };
    Form.prototype.updateForm = function () {
        if (this.updateFormTimer !== -1)
            return;
        this.updateFormTimer = setTimeout(this.batchedUpdateForm);
    };
    Form.prototype.batchedUpdateForm = function () {
        this.props.onChange(this.formState);
        this.updateFormTimer = -1;
    };
    Form.prototype.updateField = function (name, nextInputState) {
        this.formState[name] = nextInputState;
        updateFormTests(this.formState);
        /** Mutates formState */
        function updateFormTests(formState) {
            formState.isError = hasError(formState, true);
            formState.isValid = !hasError(formState);
            formState.isChanged = hasChanged(formState);
        }
        function hasError(formState, skipPristine) {
            if (skipPristine === void 0) { skipPristine = false; }
            for (var _i = 0, _a = Object.keys(formState); _i < _a.length; _i++) {
                var field = _a[_i];
                if (formState[field] !== Object(formState[field]))
                    continue; // Skip booleans and other non-objects
                var input = formState[field];
                if (skipPristine && !input.dirty)
                    continue; // Can't have error if we haven't touched the field
                if (input.honestError !== 0)
                    return true;
            }
            return false;
        }
        function hasChanged(formState) {
            for (var _i = 0, _a = Object.keys(formState); _i < _a.length; _i++) {
                var field = _a[_i];
                if (formState[field] !== Object(formState[field]))
                    continue; // Skip booleans and other non-objects
                var input = formState[field];
                if (input.changed)
                    return true;
            }
            return false;
        }
    };
    Form.childContextTypes = {
        updateField: PropTypes.func,
        updateForm: PropTypes.func,
    };
    return Form;
}(React.Component));
exports.Form = Form;
