import * as React from "react";
import * as PropTypes from "prop-types";
import { InputState } from "./InputState";
export interface FieldProps extends React.HTMLProps<void> {
    value: any;
    name: string;
    onChange?: (value: any) => void;
    debounce?: "onBlur" | number;
    validate?: (value: any) => number;
    /** If type is number, the numeric value must be >= min. Else, the string length must be >= to min*/
    min?: number;
    /** If type is number, the numeric value must be <= max. Else, the string length must be <= to max */
    max?: number;
    /** Input must match regex expression1 */
    regex?: string | RegExp;
    /** Must have a value or Field is invalid */
    required?: boolean;
    /** Remove whitespace (left and right) from input */
    trimWhitespace?: boolean;
}
export interface FieldState {
    originalValue: any;
    value: any;
    input: InputState<any>;
}
/**
 * Field wraps around a single! input component that implements the onChange/value controlled component contract.
 * Must be inside a <Form> component.
 */
export declare class Field extends React.Component<FieldProps, FieldState> {
    static contextTypes: {
        updateField: PropTypes.Requireable<any>;
        updateForm: PropTypes.Requireable<any>;
    };
    private impl;
    private debounceTimer;
    constructor(props: FieldProps);
    render(): React.ReactElement<any>;
    componentWillMount(): void;
    componentDidMount(): void;
    protected onBlur(): void;
    private handleChange(state);
    private updateForm();
}
