"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
var InvalidField_1 = require("./impl/InvalidField");
var CustomField_1 = require("./impl/CustomField");
var TextInputField_1 = require("./impl/input/TextInputField");
var NumberInputField_1 = require("./impl/input/NumberInputField");
var EmailInputField_1 = require("./impl/input/EmailInputField");
var CheckboxInputField_1 = require("./impl/input/CheckboxInputField");
var RadioButtonInputField_1 = require("./impl/input/RadioButtonInputField");
var SelectinputField_1 = require("./impl/input/SelectinputField");
/**
 * Field wraps around a single! input component that implements the onChange/value controlled component contract.
 * Must be inside a <Form> component.
 */
var Field = (function (_super) {
    __extends(Field, _super);
    function Field(props) {
        var _this = _super.call(this, props) || this;
        _this.debounceTimer = -1;
        _this.handleChange = _this.handleChange.bind(_this);
        _this.updateForm = _this.updateForm.bind(_this);
        _this.onBlur = _this.onBlur.bind(_this);
        _this.state = {
            originalValue: props.value,
            value: props.value,
            input: undefined,
        };
        return _this;
    }
    Field.prototype.render = function () {
        return this.impl.render();
    };
    Field.prototype.componentWillMount = function () {
        var _this = this;
        // Check for error with the children. Return a render that renders null if errors.
        var impl = validityTest(this.props);
        if (impl) {
            this.impl = impl;
            return;
        }
        var getProps = function () { return _this.props; };
        var getState = function () { return _this.state; };
        var getContext = function () { return _this.context; };
        // Check if a valid radio button was found
        if (this.props.children instanceof Array && this.props.children.length > 1) {
            this.impl = new RadioButtonInputField_1.RadioButtonInputField(this.handleChange, this.onBlur, getProps, getState);
        }
        else {
            var child = React.Children.only(this.props.children);
            if (child.type === "input") {
                switch (child.props.type) {
                    case "number":
                        this.impl = new NumberInputField_1.NumberInputField(this.handleChange, this.onBlur, getProps, getState);
                        break;
                    case "search":
                    case "text":
                    case "password":
                        this.impl = new TextInputField_1.TextInputField(this.handleChange, this.onBlur, getProps, getState);
                        break;
                    case "email":
                        this.impl = new EmailInputField_1.EmailInputField(this.handleChange, this.onBlur, getProps, getState);
                        break;
                    case "checkbox":
                        this.impl = new CheckboxInputField_1.CheckboxInputField(this.handleChange, this.onBlur, getProps, getState);
                        break;
                    default:
                        this.impl = new CustomField_1.CustomField(this.handleChange, getProps, getState, getContext);
                        break;
                }
            }
            else if (child.type === "select") {
                this.impl = new SelectinputField_1.SelectInputField(this.handleChange, this.onBlur, getProps, getState);
            }
            else {
                this.impl = new CustomField_1.CustomField(this.handleChange, getProps, getState, getContext);
            }
        }
        function validityTest(props) {
            var impl = undefined;
            if (props.children instanceof Array && props.children.length > 1) {
                // Only time it is valid to have multiple children is when they belong to a radio button
                for (var i = 0; i < props.children.length; i++) {
                    var child = props.children[i];
                    if (!child || child.type === "label")
                        continue;
                    if (child.type !== "input" || child.props.type !== "radio") {
                        impl = new InvalidField_1.InvalidField();
                    }
                }
                if (impl) {
                    console.warn("No more than one child is allowed inside a <Field>");
                }
            }
            else if (props.children == null) {
                console.warn("Need child component inside a <Field>");
                impl = new InvalidField_1.InvalidField();
            }
            return impl;
        }
    };
    Field.prototype.componentDidMount = function () {
        var nextState = this.impl.componentDidMount();
        this.setState(nextState);
        this.context.updateField(this.props.name, nextState.input);
    };
    Field.prototype.onBlur = function () {
        if (this.props.debounce === "onBlur") {
            this.updateForm();
        }
    };
    Field.prototype.handleChange = function (state) {
        this.setState(state);
        this.context.updateField(this.props.name, state.input);
        // Check for debounce. If numeric then install timeout. if onBlur, just exit function without updating Form.
        clearTimeout(this.debounceTimer);
        if (this.props.debounce) {
            if (this.props.debounce > 0)
                this.debounceTimer = setTimeout(this.updateForm, this.props.debounce);
        }
        else {
            this.updateForm();
        }
    };
    Field.prototype.updateForm = function () {
        if (this.props.onChange)
            this.props.onChange(this.state.value);
        this.context.updateForm();
    };
    Field.contextTypes = {
        updateField: PropTypes.func,
        updateForm: PropTypes.func,
    };
    return Field;
}(React.Component));
exports.Field = Field;
