"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var helpers_1 = require("../helpers");
var CustomField = (function () {
    function CustomField(onChange, getProps, getState, getContext, originalValue, name) {
        this.onChange = onChange;
        this.getProps = getProps;
        this.getState = getState;
        this.getContext = getContext;
        this.originalValue = originalValue;
        this.name = name;
        this.handleChange = this.handleChange.bind(this);
        var state = getState();
        var input = this.nextInput(state.value, true);
        var nextState = __assign({}, state, { value: state.value, input: input });
        this.onChange(nextState);
    }
    CustomField.prototype.render = function () {
        var props = this.getProps(), state = this.getState();
        var child = React.Children.only(props.children);
        var onChange = this.handleChange;
        return React.cloneElement(child, {
            value: state.value,
            onChange: onChange,
        });
    };
    CustomField.prototype.componentDidUpdate = function (prevProps, prevState) {
        var props = this.getProps(), state = this.getState();
        if (state.input !== undefined &&
            (prevState === undefined || prevState.input !== state.input)) {
            var willUpdateAsync = helpers_1.isAsync(props);
            if (willUpdateAsync)
                return;
            this.updateField();
        }
    };
    CustomField.prototype.propsUpdate = function () {
        var state = this.getState();
        var input = this.nextInput(state.value, state.input.dirty);
        if (input.value !== state.input.value ||
            input.error !== state.input.error ||
            input.dirty !== state.input.dirty ||
            input.changed !== state.input.changed ||
            input.honestError !== state.input.honestError) {
            var nextState = __assign({}, state, { value: input.value, input: input });
            this.onChange(nextState);
        }
    };
    CustomField.prototype.handleChange = function (value) {
        var state = this.getState();
        var input = this.nextInput(value, true);
        var nextState = __assign({}, state, { value: value, input: input });
        this.onChange(nextState);
    };
    CustomField.prototype.updateField = function () {
        var props = this.getProps(), state = this.getState(), context = this.getContext();
        if (props.onChange)
            props.onChange(state.value);
        context.updateForm(this.name, state.input);
    };
    CustomField.prototype.nextInput = function (value, dirty) {
        var props = this.getProps();
        var honestError = props.validate ? props.validate(value) : 0;
        return {
            name: this.name,
            value: value,
            dirty: dirty,
            honestError: honestError,
            error: dirty ? honestError : 0,
            changed: value !== this.originalValue,
        };
    };
    return CustomField;
}());
exports.CustomField = CustomField;
