"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var FormError_1 = require("../../FormError");
var InputField = (function () {
    function InputField(onChange, getProps, getState, getContext) {
        this.onChange = onChange;
        this.getProps = getProps;
        this.getState = getState;
        this.getContext = getContext;
        this.handleInputChange = this.handleInputChange.bind(this);
        this.handleInputOnBlur = this.handleInputOnBlur.bind(this);
    }
    InputField.prototype.render = function () {
        var props = this.getProps(), state = this.getState();
        var child = React.Children.only(props.children);
        var onBlur = (props.debounce !== undefined && props.debounce === "onBlur") ? this.handleInputOnBlur : undefined;
        var onChange = this.handleInputChange;
        if (child.type === "input" && child.props.type === "checkbox") {
            return React.cloneElement(child, {
                checked: state.value,
                id: props.name,
                name: props.name,
                onChange: onChange
            });
        }
        else {
            return React.cloneElement(child, {
                value: state.value || "",
                id: props.name,
                name: props.name,
                onBlur: onBlur,
                onChange: onChange
            });
        }
    };
    InputField.prototype.componentDidMount = function () {
        var props = this.getProps(), state = this.getState();
        var input = this.nextInput(state.value, props.children.props.type, false);
        return { input: input };
    };
    InputField.prototype.componentDidUpdate = function (prevProps, prevState) { };
    InputField.prototype.handleInputOnBlur = function () {
        var props = this.getProps(), state = this.getState(), context = this.getContext();
        context.updateFormState(props.name, state.input);
    };
    InputField.prototype.checkErrors = function (value, type) {
        var props = this.getProps();
        if (props.required && (value == null || value.length === 0))
            return FormError_1.FormError.ValueRequired;
        if (!props.required && (value == null || value.length === 0))
            return 0;
        if (props.min) {
            if (type === "number") {
                var conv = +value;
                if (conv < props.min)
                    return FormError_1.FormError.MinLengthNotMet;
            }
            else {
                if (value.length < props.min)
                    return FormError_1.FormError.MinLengthNotMet;
            }
        }
        if (props.max) {
            if (type === "number") {
                var conv = +value;
                if (conv > props.max)
                    return FormError_1.FormError.MaxLengthExceeded;
            }
            else {
                if (value.length > props.max)
                    return FormError_1.FormError.MaxLengthExceeded;
            }
        }
        if (type === "email") {
            var match = value.match(InputField.emailRegex);
            if (match == null || match.length === 0)
                return FormError_1.FormError.InvalidEmail;
        }
        if (props.regex) {
            var regexp = typeof props.regex === "string" ? new RegExp(props.regex) : props.regex;
            var match = value.match(regexp);
            if (match == null || match.length === 0)
                return FormError_1.FormError.InvalidRegex;
        }
        if (props.validate)
            return props.validate(value);
        return 0;
    };
    InputField.prototype.handleInputChange = function (e) {
        var state = this.getState();
        var type = e.currentTarget.type;
        var nextState;
        if (type === "checkbox") {
            var value = !state.value;
            var input = this.nextInput(value, type, true);
            nextState = __assign({}, state, { value: value, input: input });
        }
        else {
            var value = e.currentTarget.value;
            var input = this.nextInput(value, type, true);
            nextState = __assign({}, state, { value: value, input: input });
        }
        this.onChange(nextState);
    };
    InputField.prototype.nextInput = function (value, type, dirty) {
        var props = this.getProps(), state = this.getState();
        var honestError = type === "checkbox" ? ((props.required && !value) ? FormError_1.FormError.ValueRequired : 0) : this.checkErrors(value, type);
        return {
            name: props.name,
            value: value,
            dirty: dirty,
            honestError: honestError,
            error: dirty ? honestError : 0,
            changed: value !== state.originalValue,
        };
    };
    InputField.emailRegex = (/^.+@.+$/g);
    return InputField;
}());
exports.InputField = InputField;
