"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var FormError_1 = require("../FormError");
var emailRegex = (/^.+@.+$/g);
function checkTextRequired(props, value) {
    // missing a required field
    return (props.required && (value == null || value === undefined || value.length === 0)) ?
        FormError_1.FormError.ValueRequired : 0;
}
exports.checkTextRequired = checkTextRequired;
function checkNumberRequired(props, value) {
    // missing a required field
    return (props.required && (value === null || isNaN(value))) ?
        FormError_1.FormError.ValueRequired : 0;
}
exports.checkNumberRequired = checkNumberRequired;
function checkCheckboxRequired(props, value) {
    // missing a required field
    return (props.required && (value == null || value === undefined || value === false)) ?
        FormError_1.FormError.ValueRequired : 0;
}
exports.checkCheckboxRequired = checkCheckboxRequired;
function checkNumericBounds(props, value) {
    // If input is number type, then min is treated as numeric bound. Else as length check
    if (props.min) {
        if (value < props.min)
            return FormError_1.FormError.MinLengthNotMet;
    }
    // If input is number type, then max is treated as numeric bounds. Else as length check
    if (props.max) {
        if (value > props.max)
            return FormError_1.FormError.MaxLengthExceeded;
    }
    return 0;
}
exports.checkNumericBounds = checkNumericBounds;
function checkStringBounds(props, value) {
    // If input is number type, then min is treated as numeric bound. Else as length check
    if (props.min) {
        if (value.length < props.min)
            return FormError_1.FormError.MinLengthNotMet;
    }
    // If input is number type, then max is treated as numeric bounds. Else as length check
    if (props.max) {
        if (value.length > props.max)
            return FormError_1.FormError.MaxLengthExceeded;
    }
    return 0;
}
exports.checkStringBounds = checkStringBounds;
function checkEmail(props, value) {
    if (value.length === 0)
        return 0; // No errors if no input
    var match = value.match(emailRegex);
    if (match == null || match.length === 0)
        return FormError_1.FormError.InvalidEmail;
    return 0;
}
exports.checkEmail = checkEmail;
function checkRegex(props, value) {
    if (value === null || value.length === 0)
        return 0; // No errors if no input
    if (props.regex) {
        var regexp = typeof props.regex === "string" ? new RegExp(props.regex) : props.regex;
        var match = value.match(regexp);
        if (match == null || match.length === 0)
            return FormError_1.FormError.InvalidRegex;
    }
    return 0;
}
exports.checkRegex = checkRegex;
function checkValidate(props, value) {
    if (props.validate)
        return props.validate(value);
    return 0;
}
exports.checkValidate = checkValidate;
