import * as React from "react";
import {BaseForm, Field, Form, formState, InputState} from "react-informed";

export interface CsfUserInfoForm extends BaseForm {
    csfFirstName: InputState<string>,
    csfLastName: InputState<string>;
    csfEmail: InputState<string>;
    csfHowMany: InputState<number>;
    csfCheckbox: InputState<boolean>;
    csfFile: InputState<string>;
}

export interface ClientSideFormState {
    form: CsfUserInfoForm;
}

export class ClientSideForm extends React.Component<object, ClientSideFormState> {
    constructor(props: object) {
        super(props);

        this.handleFormChanged = this.handleFormChanged.bind(this);
        this.handleFileSelected = this.handleFileSelected.bind(this);

        this.state = {
            form: formState({
                csfFirstName: "",
                csfLastName: "",
                csfEmail: "",
                csfFile: "",
                csfHowMany: "",
                csfCheckbox: false,
            }),
        };
    }

    public render() {
        let { form } = this.state;
        return (
            <div>
                <Form formState={form}
                      preventAction={true}
                      className="form-inline"
                      onChange={this.handleFormChanged}>

                    <div>
                        <label htmlFor={form.csfFirstName.name}>
                            First Name (regex="^[A-Z ]+$", min=1, max=10, trimWhitespace)
                        </label>
                        <Field value={form.csfFirstName}
                               regex="^[A-Z ]+$"
                               min={1}
                               max={10}
                               trimWhitespace>
                            <input type="text"
                                   placeholder="firstName"
                            />
                        </Field>
                    </div>

                    <div className="col-xs-12 col-md-6">
                        <label htmlFor={form.csfLastName.name}>Last Name (regex=/^[A-C][a-z]+$/)</label>
                        <Field value={form.csfLastName} regex={/^[A-C][a-z]+$/}>
                            <input type="text"
                                   placeholder="lastName"
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.csfEmail.name}>Email</label>
                        <Field value={form.csfEmail}>
                            <input type="email"
                                   placeholder="email"
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.csfHowMany.name}>
                            howMany(max=2 when checkbox is selected. Else max=5, required)
                        </label>
                        <Field value={form.csfHowMany}
                               max={form.csfCheckbox.value ? 2 : 5}
                               required>
                            <input type="number"
                                   placeholder="number"
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.csfCheckbox.name}>Checkbox (required)</label>
                        <Field value={form.csfCheckbox} required>
                            <input type="checkbox"/>
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.csfFile.name}>File</label>
                        <Field value={form.csfFile} onChange={this.handleFileSelected}>
                            <input type="file"/>
                        </Field>
                    </div>
                    {form.csfEmail.error > 0 && (
                        <div style={{ color: "#D33" }}>
                            Email is wrong, y'know?
                        </div>)
                    }
                    {form.csfCheckbox.error > 0 && (
                        <div style={{ color: "#D33" }}>
                            The checkbox is required.
                        </div>)
                    }
                    <div>
                        <button disabled={(!form.isValid || form.isSubmitted) || !form.isChanged}
                                className="btn btn-primary"
                                type="submit">
                            submit
                        </button>
                    </div>
                </Form>

                <pre>
                    {JSON.stringify(this.state.form, null, 4)}
                </pre>
            </div>);
    }

    private handleFormChanged(form: CsfUserInfoForm) {
        this.setState({ form });
    }

    private handleFileSelected(event: React.FormEvent<HTMLInputElement>) {
        // Do any operations needed for files here.
    }
}

