import * as React from "react";
import {BaseForm, Field, Form, formState, InputState} from "react-informed";
import {CustomComponent, CustomObject, CustomObjectError} from "./CustomComponent";

export interface CsfUserInfoForm extends BaseForm {
    ccfName: InputState<string>,
    ccfCustom: InputState<CustomObject>;
}

export interface CustomComponentFormState {
    form: CsfUserInfoForm;
}

export class CustomComponentForm extends React.Component<any, CustomComponentFormState> {
    constructor(props: any) {
        super(props);

        this.handleFormChanged = this.handleFormChanged.bind(this);
        this.validateCustomComponent = this.validateCustomComponent.bind(this);
        this.state = {
            form: formState({
                ccfName: "",
                ccfCustom: { id: 1, message: "12345" },
            }),
        };
    }

    public render() {
        let { form } = this.state;
        return (
            <div>
                <Form formState={form}
                      preventAction={true}
                      onChange={this.handleFormChanged}>

                    <div>
                        <label htmlFor={form.ccfName.name}>Name</label>
                        <Field value={form.ccfName}>
                            <input type="text"
                                   placeholder="Names"
                            />
                        </Field>
                    </div>


                    <div>
                        <label htmlFor={form.ccfCustom.name}>
                            Custom Component(Must be a number, max length=5)
                        </label>
                        <Field value={form.ccfCustom} validate={this.validateCustomComponent}>
                            <CustomComponent/>
                        </Field>
                    </div>

                    <div>
                        <button disabled={!form.isValid || form.isSubmitted}
                                className="btn btn-primary"
                                type="submit">
                            submit
                        </button>
                    </div>
                </Form>

                <pre>
                    {JSON.stringify(this.state.form, null, 4)}
                </pre>
            </div>);
    }

    private handleFormChanged(form: CsfUserInfoForm) {
        this.setState({ form });
    }

    private validateCustomComponent(value: CustomObject): CustomObjectError {
        if (value.message.length > 5) return CustomObjectError.MessageIsTooLong;
        if (isNaN(+value.message)) return CustomObjectError.MessageIsNumber;
        return 0;
    }
}

