import * as React from "react";
import {BaseForm, Field, Form, formState, InputState} from "react-informed";

export interface UserInfoForm extends BaseForm {
    firstName: InputState<string>,
    lastName: InputState<string>;
    password: InputState<string>;
    email: InputState<string>;
    select: InputState<string>;
    checkbox: InputState<boolean>;
    radiobutton: InputState<string>;
    date: InputState<string>;
}

export interface SimpleFormState {
    form: UserInfoForm;
}

export class SimpleForm extends React.Component<any, SimpleFormState> {
    private static radiobuttonOptions = ["purple", "pink", "blue"];

    constructor(props: any) {
        super(props);

        this.handleFormChanged = this.handleFormChanged.bind(this);
        this.resetSubmitted = this.resetSubmitted.bind(this);

        this.state = {
            form: formState({
                firstName: "krillin",
                lastName: "space duck",
                password: "quack",
                email: "",
                select: "",
                checkbox: false,
                radiobutton: "pink",
                date: "2017-08-22",
            }),
        };
    }

    public render() {
        let { form } = this.state;
        let radiobuttonOptions = SimpleForm.radiobuttonOptions;
        return (
            <div>
                <Form formState={form}
                      preventAction={true}
                      onChange={this.handleFormChanged}>

                    <div>
                        <label htmlFor={form.firstName.name}>First Name</label>
                        <Field value={form.firstName}>
                            <input type="text"
                                   placeholder="firstName"
                                   disabled={form.isSubmitted}
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.lastName.name}>Last Name (updates when input loses focus)</label>
                        <Field value={form.lastName} onBlur>
                            <input type="text"
                                   placeholder="lastName"
                                   disabled={form.isSubmitted}
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.password.name}>Password</label>
                        <Field value={form.password}>
                            <input type="password"
                                   disabled={form.isSubmitted}
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.email.name}>Email(updates after 2000 ms or onBlur)</label>
                        <Field value={form.email}
                               debounce={2000}>
                            <input type="email"
                                   placeholder="email"
                                   disabled={form.isSubmitted}
                            />
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.select.name}>Select</label>
                        <Field value={form.select}>
                            <select disabled={form.isSubmitted}>
                                <option disabled hidden value="">Select an option</option>
                                <option value="0">Blue</option>
                                <option value="1">Red</option>
                                <option value="2">Option 3</option>
                            </select>
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.checkbox.name}>Checkbox</label>
                        <Field value={form.checkbox}>
                            <input type="checkbox" disabled={form.isSubmitted}/>
                        </Field>
                    </div>

                    <div>
                        <Field value={form.radiobutton}>
                            <label htmlFor={radiobuttonOptions[0]}>{radiobuttonOptions[0]}</label>
                            <input type="radio"
                                   value={radiobuttonOptions[0]}
                                   disabled={form.isSubmitted}/>
                            <label htmlFor={radiobuttonOptions[1]}>{radiobuttonOptions[1]}</label>
                            <input type="radio"
                                   value={radiobuttonOptions[1]}
                                   disabled={form.isSubmitted}/>
                            <label htmlFor={radiobuttonOptions[2]}>{radiobuttonOptions[2]}</label>
                            <input type="radio"
                                   value={radiobuttonOptions[2]}
                                   disabled={form.isSubmitted}/>
                        </Field>
                    </div>

                    <div>
                        <label htmlFor={form.date.name}>Date</label>
                        <Field value={form.date}>
                            <input type="date" disabled={form.isSubmitted}/>
                        </Field>
                    </div>

                    <div>
                        <button disabled={form.isSubmitted}
                                className="btn btn-primary"
                                type="submit">
                            submit
                        </button>
                    </div>
                </Form>

                <button onClick={this.resetSubmitted}>Reset submitted property</button>

                <pre>
                    {JSON.stringify(this.state.form, null, 4)}
                </pre>
            </div>);
    }

    private handleFormChanged(form: UserInfoForm) {
        this.setState({ form });
    }

    private resetSubmitted() {
        this.setState({ form: { ...this.state.form, isSubmitted: false } });
    }
}


